package br.com.eselo.controller;

import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.Messages;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.ConformidadeDao;
import br.com.eselo.dao.DiretrizDao;
import br.com.eselo.dao.ParametroAvaliacaoDao;
import br.com.eselo.dao.UsuarioDao;
import br.com.eselo.model.Conformidade;
import br.com.eselo.model.Diretriz;
import br.com.eselo.model.ParametroAvaliacao;
import br.com.eselo.model.Usuario;

@Resource
public class ConformidadesController {

	private final ConformidadeDao dao;
	private final DiretrizDao daoDiretriz;
	private final ParametroAvaliacaoDao daoParametro;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;

	public ConformidadesController(ConformidadeDao _dao,
			DiretrizDao _daoDiretriz, ParametroAvaliacaoDao _daoParametro,
			Result _result, Validator _validator, UsuarioWeb _usuarioWeb) {
		dao = _dao;
		daoDiretriz = _daoDiretriz;
		daoParametro = _daoParametro;
		result = _result;
		validator = _validator;
		usuarioWeb = _usuarioWeb;
	}

	@Get("/conformidades")
	@Restrito(descricao = "CONFORMIDADES_LISTA")
	public List<Conformidade> lista() {
		return dao.listAll();
	}

	@Get("/conformidades/gerarForm/{diretrizId}/{quantidade}")
	@Restrito(descricao = "CONFORMIDADES_ADICIONAR")
	public void gerarForm(Long diretrizId, Long quantidade) {
		result.include("quantidadeConformidades", quantidade);
		result.forwardTo(this).add(diretrizId);
	}

	@Get("/conformidades/{diretrizId}/")
	@Restrito(descricao = "CONFORMIDADES_ADICIONAR")
	public void add(Long diretrizId) {
		Diretriz diretriz = daoDiretriz.load(diretrizId);
		result.include("diretrizId", diretrizId);
		result.include("diretriz", diretriz);
		populateSelectLists();
	}

	@Get("/conformidades/{diretrizId}/{id}")
	@Restrito(descricao = "CONFORMIDADES_ATUALIZAR")
	public Conformidade form(Long diretrizId, Long id) {
		Conformidade conformidade = dao.load(id);

		Diretriz diretriz = daoDiretriz.load(diretrizId);
		ParametroAvaliacao parametro = daoParametro.load(diretriz
				.getParametroAvaliacao().getId());

		result.include("diretrizId", diretrizId);
		result.include("parametroAvaliacao", parametro);
		populateSelectLists();

		return conformidade;
	}

	@Post("/conformidades/add")
	@Restrito(descricao = "CONFORMIDADES_ADICIONAR")
	public void adicionar(List<Conformidade> conformidades) {
		Long diretrizId = null;
		for (Conformidade conformidade : conformidades) {
			diretrizId = conformidade.getDiretriz().getId();
			validarConformidade(conformidade);

			validator.onErrorUsePageOf(ConformidadesController.class).add(
					conformidade.getDiretriz().getId());
		}
		
		dao.saveAll(conformidades);

		Diretriz diretriz = daoDiretriz.load(diretrizId);

		result.include("mensagem",
				Messages.getString("conformidades.mensagem.salva"));
		result.redirectTo(DiretrizController.class).form(
				diretriz.getParametroAvaliacao().getId(), diretriz.getId());
	}

	@Put("/conformidades")
	@Restrito(descricao = "CONFORMIDADES_ATUALIZAR")
	public void atualizar(Conformidade conformidade) {
		Conformidade conformidadeCadastrada = dao.load(conformidade.getId());

		validarConformidade(conformidade);

		populateSelectLists();

		validator.onErrorUsePageOf(ConformidadesController.class).form(
				conformidadeCadastrada.getDiretriz().getId(),
				conformidadeCadastrada.getId());

		conformidadeCadastrada.setDiretriz(conformidade.getDiretriz());
		conformidadeCadastrada.setFaixaInicio(conformidade.getFaixaInicio());
		conformidadeCadastrada.setFaixaFim(conformidade.getFaixaFim());

		ParametroAvaliacao ultimoParametroAvaliacao = daoDiretriz
				.getLastParametrosAvaliacao();

		Diretriz diretriz = daoDiretriz
				.load(conformidade.getDiretriz().getId());

		if (daoDiretriz.hasAvaliacao(ultimoParametroAvaliacao)) {
			criarNovoParametroAvaliacao(diretriz, conformidade);
		} else {
			dao.saveOrUpdate(conformidadeCadastrada);
		}

		result.include("mensagem",
				Messages.getString("conformidades.mensagem.alterada"));
		result.redirectTo(DiretrizController.class).form(
				diretriz.getParametroAvaliacao().getId(), diretriz.getId());
	}

	@Path("/conformidades/delete/{id}")
	@Restrito(descricao = "CONFORMIDADES_EXCLUIR")
	public void excluir(Long id) {
		Conformidade conformidade = dao.load(id);

		ParametroAvaliacao ultimoParametroAvaliacao = daoDiretriz
				.getLastParametrosAvaliacao();

		Diretriz diretriz = daoDiretriz
				.load(conformidade.getDiretriz().getId());

		if (daoDiretriz.hasAvaliacao(ultimoParametroAvaliacao)) {
			criarNovoParametroAvaliacao(diretriz, conformidade);
		} else {
			dao.remove(id);
		}

		result.include("mensagem", "Conformidade excluída com sucesso.");
		result.redirectTo(DiretrizController.class).form(
				diretriz.getParametroAvaliacao().getId(), diretriz.getId());
	}

	private void populateSelectLists() {
		result.include("diretrizList", dao.getAllDiretriz());
	}

	private void validarConformidade(Conformidade conformidade) {
		if (conformidade.getDiretriz().getId() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("conformidades.validacao.diretriz"),
					"conformidade.diretriz.id"));
		} else if (conformidade.getFaixaInicio() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("conformidades.validacao.faixa_inicial"),
					"conformidade.diretriz.id"));
		} else if (conformidade.getFaixaFim() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("conformidades.validacao.faixa_final"),
					"conformidade.diretriz.id"));
		} else if (conformidade.getFaixaInicio() > conformidade.getFaixaFim()) {
			validator.add(new ValidationMessage(Messages
					.getString("conformidades.validacao.range_faixas"),
					"conformidade.diretriz.id"));
		}
	}

	private void criarNovoParametroAvaliacao(Diretriz diretriz,
			Conformidade conformidade) {
		ParametroAvaliacao ultimoParametroAvaliacao = daoDiretriz
				.getLastParametrosAvaliacao();

		UsuarioDao uDao = new UsuarioDao(dao.getSession());
		Usuario usuario = uDao.load(usuarioWeb.getId());

		// Cria o parametro
		ParametroAvaliacao parametroAvaliacao = new ParametroAvaliacao();
		parametroAvaliacao.setUsuario(usuario);
		parametroAvaliacao.setTsInicio(new java.util.Date(System
				.currentTimeMillis()));
		parametroAvaliacao.setTsFim(null);

		ultimoParametroAvaliacao.setTsFim(parametroAvaliacao.getTsInicio());

		ParametroAvaliacaoDao paDao = new ParametroAvaliacaoDao(
				dao.getSession());

		// Cria a lista das diretrizes
		List<Diretriz> listaCadastrada = daoDiretriz
				.diretrizesDoUltimoParametro(ultimoParametroAvaliacao);

		// Salva novo e atualiza o ultimo
		paDao.save(parametroAvaliacao);
		paDao.update(ultimoParametroAvaliacao);

		// Salva as diretrizes para nova avaliação
		Diretriz novaDiretriz;
		Conformidade novaConformidade;
		for (Diretriz diretrizCadastrada : listaCadastrada) {
			novaDiretriz = new Diretriz();
			novaDiretriz.setParametroAvaliacao(parametroAvaliacao);
			novaDiretriz.setDescricao(diretrizCadastrada.getDescricao());
			novaDiretriz.setPeso(diretrizCadastrada.getPeso());
			novaDiretriz.setNumeroDiretriz(diretrizCadastrada
					.getNumeroDiretriz());
			novaDiretriz.setTipoDiretriz(diretrizCadastrada.getTipoDiretriz());

			daoDiretriz.save(novaDiretriz);

			if (diretrizCadastrada.getId().equals(diretriz.getId())) {
				conformidade.setDiretriz(novaDiretriz);
				dao.save(conformidade);
			}

			for (Conformidade conformidadeSalva : diretrizCadastrada
					.getConformidades()) {
				novaConformidade = new Conformidade();
				novaConformidade.setDiretriz(novaDiretriz);
				novaConformidade.setFaixaInicio(conformidadeSalva
						.getFaixaInicio());
				novaConformidade.setFaixaFim(conformidadeSalva.getFaixaFim());
				dao.save(novaConformidade);
			}
		}
	}

}
