package br.com.eselo.controller;

import java.util.ArrayList;
import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.DiretrizDao;
import br.com.eselo.dao.ParametroAvaliacaoDao;
import br.com.eselo.dao.UsuarioDao;
import br.com.eselo.model.Diretriz;
import br.com.eselo.model.ParametroAvaliacao;
import br.com.eselo.model.Usuario;

@Resource
public class DiretrizController {

	private final DiretrizDao dao;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;

	private static final int MODO_ADD_DIRETRIZ = 1;
	private static final int MODO_EDIT_DIRETRIZ = 2;
	private static final int MODO_DELETE_DIRETRIZ = 3;

	public DiretrizController(DiretrizDao _dao, Result _result,
			Validator _validator, UsuarioWeb _usuarioWeb) {
		dao = _dao;
		result = _result;
		validator = _validator;
		usuarioWeb = _usuarioWeb;
	}

	@Get("/diretriz")
	@Restrito(descricao = "DIRETRIZ_LISTA")
	public List<Diretriz> lista() {
		return this.index(1);
	}

	@Get("/diretriz/{pagina}")
	@Restrito(descricao = "DIRETRIZ_LISTA")
	public List<Diretriz> index(int pagina) {
		ParametroAvaliacao ultimoParametroAvaliacao = dao
				.getLastParametrosAvaliacao();

		// Paginação
		// Quantidade de registros por pagina
		int exibi = 7;

		// Informa a quantidade de registros
		int registros = dao.numLinhas(ultimoParametroAvaliacao);

		// Quantidade de paginas na paginação
		int totalPag = registros / exibi;

		// Se houver resto na divisao acima acrescenta uma pagina na paginação
		if ((registros - (totalPag * exibi)) > 0)
			totalPag++;

		// Calculo do registro inicial e final
		int regIni = ((exibi * pagina) - exibi);
		int regFim = (regIni + exibi);

		if (regFim > registros) {
			regFim = registros;
		}

		// Fim paginação
		List<Diretriz> resultado = dao.listAllAtivas(ultimoParametroAvaliacao);
		List<Diretriz> paginacao = new ArrayList<>();

		for (int i = regIni; i < regFim; i++) {
			paginacao.add(resultado.get(i));
		}

		result.include("paginas", totalPag);
		result.include("registros", registros);
		result.include("regInic", ++regIni);
		result.include("regFim", regFim);
		result.include("controle", "diretriz");
		result.include("parametroAvaliacao", ultimoParametroAvaliacao);

		return paginacao;
	}

	@Get("/diretriz/form/{parametroId}/{id}")
	@Restrito(descricao = "DIRETRIZ_ATUALIZAR")
	public Diretriz form(Long parametroId, Long id) {
		Diretriz diretriz = this.dao.load(id);
		result.include("parametroAvaliacaoId", parametroId);
		result.include("parametroAvaliacao", diretriz.getParametroAvaliacao());
		populateSelectLists();
		
		return diretriz;
	}

	@Get("/diretriz/form/{parametroId}")
	@Restrito(descricao = "DIRETRIZ_ADICIONAR")
	public void form(Long parametroId) {
		result.include("parametroAvaliacaoId", parametroId);
		populateSelectLists();
	}

	@Post("/diretriz")
	@Restrito(descricao = "DIRETRIZ_ADICIONAR")
	public void adicionar(Diretriz diretriz) {
		if (diretriz.getNumeroDiretriz() == null) {
			validator.add(new ValidationMessage(
					"É obrigatório informar o número da diretriz.",
					"diretriz.numeroDiretriz"));
		} else if (hasNumeroDiretrizNoParametroAtual(diretriz)) {
			validator
					.add(new ValidationMessage(
							"Já existe uma diretriz ativa com esse numero. Diretriz não cadastrada!",
							"diretriz.numeroDiretriz"));
		} else if (diretriz.getDescricao() == null) {
			validator.add(new ValidationMessage(
					"É obrigatório informar a descrição da diretriz.",
					"diretriz.descricao"));
		} else if (diretriz.getPeso() == null) {
			validator.add(new ValidationMessage(
					"É obrigatório informar o peso da diretriz.",
					"diretriz.peso"));
		} else if (diretriz.getPeso() <= 0) {
			validator.add(new ValidationMessage(
					"O peso da diretriz deve ser maior que zero.",
					"diretriz.peso"));
		}

		validator.onErrorUsePageOf(DiretrizController.class).form(
				diretriz.getParametroAvaliacao().getId());

		ParametroAvaliacao ultimoParametroAvaliacao = dao
				.getLastParametrosAvaliacao();

		// Caso exista uma avaliação cadastrada para esse parametro é criado um
		// novo parametro, com todas as diretrizes existentes neste ultimo.
		if (dao.hasAvaliacao(ultimoParametroAvaliacao)) {
			criarNovoParametroAvaliacao(diretriz, MODO_ADD_DIRETRIZ);
		} else {
			diretriz.setParametroAvaliacao(ultimoParametroAvaliacao);
		}

		dao.save(diretriz);

		result.include("mensagem", "Diretriz salva com sucesso.");
		result.redirectTo(ParametroAvaliacaoController.class).form(
				diretriz.getParametroAvaliacao().getId());
	}

	@Put("/diretriz")
	@Restrito(descricao = "DIRETRIZ_ATUALIZAR")
	public void atualizar(Diretriz diretriz) {
		Diretriz diretrizCadastrado = dao.load(diretriz.getId());
		diretrizCadastrado.setNumeroDiretriz(diretriz.getNumeroDiretriz());
		diretrizCadastrado.setNumeroDiretriz(diretriz.getNumeroDiretriz());
		diretrizCadastrado.setDescricao(diretriz.getDescricao());

		if (diretriz.getNumeroDiretriz() == null) {
			validator.add(new ValidationMessage(
					"É obrigatório informar o número da diretriz.",
					"diretriz.numeroDiretriz"));
		} else if (hasNumeroDiretrizNoParametroAtual(diretriz)) {
			validator
					.add(new ValidationMessage(
							"Já existe uma diretriz ativa com esse numero. Diretriz não atualizada!",
							"diretriz.numeroDiretriz"));
		} else if (diretriz.getDescricao() == null) {
			validator.add(new ValidationMessage(
					"É obrigatório informar a descrição da diretriz.",
					"diretriz.descricao"));
		} else if (diretriz.getPeso() == null) {
			validator.add(new ValidationMessage(
					"É obrigatório informar o peso da diretriz.",
					"diretriz.peso"));
		}

		validator.onErrorUsePageOf(ParametroAvaliacaoController.class).form(
				diretrizCadastrado.getParametroAvaliacao().getId());

		// Caso exista uma avaliação cadastrada para esse parametro criado um
		// novo parametro, com todas as diretrizes existentes neste ultimo.
		ParametroAvaliacao ultimoParametroAvaliacao = dao
				.getLastParametrosAvaliacao();

		if (dao.hasAvaliacao(ultimoParametroAvaliacao)) {
			criarNovoParametroAvaliacao(diretriz, MODO_EDIT_DIRETRIZ);

			result.include("mensagem",
					"Novo Parâmetro de avaliação criado junto com nova Diretriz com sucesso.");

		} else {
			dao.update(diretrizCadastrado);
			result.include("mensagem", "Diretriz alterada com sucesso.");
		}

		result.redirectTo(ParametroAvaliacaoController.class).form(
				diretrizCadastrado.getParametroAvaliacao().getId());
	}

	@Path("/diretriz/delete/{id}")
	@Restrito(descricao = "DIRETRIZ_EXCLUIR")
	public void excluir(Long id) {
		Diretriz diretriz = dao.load(id);

		ParametroAvaliacao ultimoParametroAvaliacao = dao
				.getLastParametrosAvaliacao();

		if (dao.hasAvaliacao(ultimoParametroAvaliacao)) {
			criarNovoParametroAvaliacao(diretriz, MODO_DELETE_DIRETRIZ);
		} else {
			dao.remove(id);
		}

		result.include("mensagem", "Diretriz excluída com sucesso.");
		result.redirectTo(DiretrizController.class).index(1);
	}

	private void criarNovoParametroAvaliacao(Diretriz diretriz, int tipo) {
		ParametroAvaliacao ultimoParametroAvaliacao = dao
				.getLastParametrosAvaliacao();

		UsuarioDao uDao = new UsuarioDao(dao.getSession());
		Usuario usuario = uDao.load(usuarioWeb.getId());

		// Cria o parametro
		ParametroAvaliacao parametroAvaliacao = new ParametroAvaliacao();
		parametroAvaliacao.setUsuario(usuario);
		parametroAvaliacao.setTsInicio(new java.util.Date(System
				.currentTimeMillis()));
		parametroAvaliacao.setTsFim(null);

		ultimoParametroAvaliacao.setTsFim(parametroAvaliacao.getTsInicio());

		ParametroAvaliacaoDao paDao = new ParametroAvaliacaoDao(
				dao.getSession());

		// Cria a lista das diretrizes
		List<Diretriz> listaCadastrada = dao
				.diretrizesDoUltimoParametro(ultimoParametroAvaliacao);

		// Salva novo e atualiza o ultimo
		paDao.save(parametroAvaliacao);
		paDao.update(ultimoParametroAvaliacao);

		diretriz.setParametroAvaliacao(parametroAvaliacao);

		// Salva as diretrizes para nova avaliação
		Diretriz novaDiretriz;
		for (Diretriz diretrizCadastrada : listaCadastrada) {
			if ((tipo == MODO_ADD_DIRETRIZ)
					|| (diretrizCadastrada.getId().longValue() != diretriz
							.getId().longValue())) {
				novaDiretriz = new Diretriz();
				novaDiretriz.setParametroAvaliacao(parametroAvaliacao);
				novaDiretriz.setDescricao(diretrizCadastrada.getDescricao());
				novaDiretriz.setPeso(diretrizCadastrada.getPeso());
				novaDiretriz.setNumeroDiretriz(diretrizCadastrada
						.getNumeroDiretriz());
				novaDiretriz.setTipoDiretriz(diretrizCadastrada
						.getTipoDiretriz());

				dao.save(novaDiretriz);
			}
		}

		switch (tipo) {
			case MODO_EDIT_DIRETRIZ:
				novaDiretriz = new Diretriz();
				novaDiretriz.setParametroAvaliacao(parametroAvaliacao);
				novaDiretriz.setDescricao(diretriz.getDescricao());
				novaDiretriz.setNumeroDiretriz(diretriz.getNumeroDiretriz());
				novaDiretriz.setPeso(diretriz.getPeso());
				novaDiretriz.setTipoDiretriz(diretriz.getTipoDiretriz());
	
				dao.save(novaDiretriz);
				break;
	
			case MODO_DELETE_DIRETRIZ:

				break;
		default:
			break;
		}
	}

	/**
	 * Recebe uma diretriz antes de ser cadastrada e verifica se ja existe uma
	 * diretriz ativa com o mesmo numero.
	 * 
	 * @param diretriz
	 * @return boolean
	 */
	private boolean hasNumeroDiretrizNoParametroAtual(Diretriz diretriz) {
		boolean retorno = false;

		List<Diretriz> listaCadastrada = dao.diretrizesDoUltimoParametro(dao
				.getLastParametrosAvaliacao());
		for (Diretriz diretrizCadastrada : listaCadastrada) {
			if (diretrizCadastrada.getNumeroDiretriz().longValue() == diretriz
					.getNumeroDiretriz().longValue()) {
				if (diretriz.getId() != null) {
					if (diretrizCadastrada.getId().longValue() != diretriz
							.getId().longValue()) {
						return true;
					}
				} else {
					return true;
				}
			}
		}
		return retorno;
	}

	private void populateSelectLists() {
		result.include("tipoDiretrizList", dao.getAllTipoDiretriz());
	}

}
